/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.java.net
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.impl;

import it.tidalwave.actor.Collaboration;
import javax.annotation.Nonnull;
import javax.annotation.concurrent.NotThreadSafe;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.concurrent.TimeUnit;
import java.io.IOException;
import java.io.PrintWriter;
import it.tidalwave.actor.CollaborationStartedMessage;
import it.tidalwave.actor.MessageSupport;
import it.tidalwave.actor.annotation.Actor;
import it.tidalwave.actor.annotation.CollaborationListener;
import it.tidalwave.actor.annotation.Message;
import it.tidalwave.actor.annotation.MessageListener;
import it.tidalwave.integritychecker.Fingerprint;
import it.tidalwave.integritychecker.FileScanRequestMessage;
import it.tidalwave.integritychecker.FileDamageDetectedMessage;
import it.tidalwave.integritychecker.FileDiscoveredMessage;
import it.tidalwave.integritychecker.FingerprintComputedMessage;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import lombok.Cleanup;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;

/***********************************************************************************************************************
 * 
 * @stereotype Actor
 * 
 * An implementation of {@link PersistenceManagerActorSupport} which stores data to a flat file.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Actor(threadSafe=false) @NotThreadSafe @Slf4j
public class PersistenceManagerActor extends PersistenceManagerActorSupport
  {
    @Message(outOfBand=true, daemon=false) @ToString
    static class FlushRequestMessage extends MessageSupport
      {
      }
    
    private final Map<String, Object> map = new TreeMap<String, Object>();
    
    private File persistenceFile;
    
    private boolean flushPending;
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @CollaborationListener(message=FileScanRequestMessage.class)
    public void prepareForNewScan (final @Nonnull CollaborationStartedMessage message)
      {
        final FileScanRequestMessage m = ((FileScanRequestMessage)message.getCollaboration().getOriginatingMessage());
        File folder = FileUtil.toFile(m.getFolder());
        
        if (!folder.canWrite())
          {  
            folder = new File(System.getProperty("user.home"));   
          }
        
        final String fileName = new SimpleDateFormat("'fingerprints-'yyyyMMdd_HHmm'.txt'").format(new Date());
        persistenceFile = new File(folder, fileName);
        map.clear();
        log.info("Persisting to {}", persistenceFile.getAbsolutePath());
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Override
    public void store (@Nonnull FileDiscoveredMessage message)
      {
        log.debug("store({})", message);
        final String name = message.getFileObject().getNameExt();
        
        if (!map.containsKey(name))
          {
            map.put(name, "unavailable");  
            requestFlush();
          }
      }

    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Override
    public void store (@Nonnull FingerprintComputedMessage message)
      {
        log.debug("store({})", message);
        final String name = message.getFileObject().getNameExt();
        final Fingerprint fingerprint = message.getFingerprint();
        map.put(name, fingerprint);
        requestFlush();
      }

    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Override
    public void store (@Nonnull FileDamageDetectedMessage message)
      {
        log.debug("store({})", message);
        final String name = message.getFileObject().getNameExt();
        map.put(name, "broken: " + message.getCause());  
        requestFlush();
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @MessageListener
    private void flush (final @Nonnull FlushRequestMessage message)
      throws InterruptedException, IOException
      {
        log.debug("flush()");
        flushPending = false;
        final @Cleanup PrintWriter pw = new PrintWriter(persistenceFile);

        for (final Entry<String, Object> entry : map.entrySet())
          {
            if (entry.getValue() instanceof Fingerprint)
              {
                final Fingerprint fingerprint = (Fingerprint)entry.getValue();
                pw.printf("%s(%s)=%s\n", fingerprint.getType(), entry.getKey(), fingerprint.getValue());
              }
            else
              {
                pw.printf("%s=%s\n", entry.getKey(), entry.getValue());
              }
          }

        pw.close();
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    private void requestFlush()
      {
        if (!flushPending)
          {
            log.debug(">>>> requesting flush...");
            new FlushRequestMessage().sendLater(5, TimeUnit.SECONDS);
            flushPending = true;
          }
      }
  }
